"""Models of Resource, Author and Tag used to defined the RESOURCES and APPS_IN_GALLERY list."""
from typing import List, NamedTuple, Optional

_IMAGE_DICT = {
    " ": "-",
    "#": "",
}


class Tag(NamedTuple):
    """Model of a Tag"""

    name: str

    def __str__(self,):
        return self.name

    def __repr__(self,):
        return self.name

    def __hash__(self,):
        return hash(self.name)


class Author(NamedTuple):
    """Model of an Author"""

    name: str
    url: str
    github_url: str
    github_avatar_url: str

    def __str__(self,):
        return self.name

    def __repr__(self,):
        return self.name

    def to_html(self, width="20px", height="20px",) -> str:
        """## A valid HTML string with the GitHub image and GitHub url link

        Returns:
            str: A valid HTML string with the github avatar link
        """
        return (
            f'<a href="{self.github_url}" title="Author: {self.name}" target="_blank">'
            f'<img src="{self.github_avatar_url}" alt="{self.name}" '
            f'style="border-radius: 50%;width: {width};height: {height};'
            'vertical-align: text-bottom;">'
            "</img></a>"
        )


class Resource:
    """Model of a Resource

        Args:
            name (str): The name of the Resource
            url (str): The url to the resource
            thumbnail_path (str): A thumbnail image of the resource
            is_awesome (bool): Whether or not the Resource should be included in the awesome-panel
            list of awesome resources.
            tags (Optional[List[Tag]], optional): A list of Tags describing the Resource.
            Used to search for Resources. Defaults to None.
            author (Optional[Author], optional): The author of the resource. Defaults to None.
    """

    def __init__(  # pylint: disable=too-many-arguments
        self,
        name: str,
        url: str,
        thumbnail_path: str,
        is_awesome: bool,
        tags: Optional[List[Tag]] = None,
        author: Optional[Author] = None,
    ):
        self.name = name
        self.url = url
        self.thumbnail_path = thumbnail_path
        self.is_awesome = is_awesome
        if tags:
            self.tags = tags
        else:
            self.tags = []
        self.author = author

    def to_markdown_bullet(self,) -> str:
        """A markdown bullet string

        Returns:
            str: The Resource as a Markdown bullet string
        """
        result = f"- [{self.name}]({self.url})"
        if self.author:
            result += f" by [{self.author.name}]({self.author.url})"
        if self.tags:
            result += " (#" + ", #".join(sorted([tag.name for tag in self.tags])) + ")"

        return result

    def __str__(self,):
        return self.name

    def __repr__(self,):
        return f"Resource(name={self.name})"

    @property
    def screenshot_file(self,) -> str:
        """The file name of a screenshot of the resource

        Returns:
            str: The file name of screenshot of the resource
        """
        file = f"{self.name.lower()}.png"
        for (original, new,) in _IMAGE_DICT.items():
            file = file.replace(original, new,)
        return file



"""This module contains a navigation menu to be used to select between different pages"""
from typing import List, Optional, Union

import panel as pn

import awesome_panel.express as pnx


class NavigationButton(pn.widgets.Button):
    """## Navigation_Button"""

    def __init__(
        self,
        page: Union[pn.layout.Panel, pn.pane.Pane, pn.widgets.Widget,],
        page_outlet: pn.layout.ListPanel,
        **kwargs,
    ):
        """## Navigation Button to navigate between pages

        Arguments:
            page {Union[pn.layout.Panel, pn.pane.Pane, pn.pane.Widget]} -- A page to navigate to
            when the button is clicked
            page_outlet {pn.layout.ListPanel} -- The ListPanel to update when the user navigates to
            a new page
        """
        if callable(page):
            page_ = page()
        else:
            page_ = page

        if "name" not in kwargs:
            if "css_classes" in kwargs:
                kwargs["name"] = " " + page_.name
            else:
                kwargs["name"] = page_.name
        if "width" not in kwargs:
            kwargs["width"] = 165
        super().__init__(**kwargs)

        def navigate_to_page(event,):  # pylint: disable=unused-argument
            page_outlet[:] = [pnx.spinners.DefaultSpinner().center()]
            page_outlet[:] = [page_]

        self.on_click(navigate_to_page)


class NavigationMenu(pn.Column):
    "## Navigation Menu"

    def __init__(  # pylint: disable=too-many-arguments
        self,
        pages: List[Union[pn.layout.Panel, pn.pane.Pane,]],
        page_outlet: pn.layout.ListPanel,
        *args,
        css_classes: Optional[List[Optional[List[str]]]] = None,
        title: str = "Navigation",
        sizing_mode: str = "stretch_width",
        **kwargs,
    ):
        """## Navigation Menu

        A widget composed of NavigationButtons that can be used to navigate between pages.

        Arguments:
            pages {List[Union[pn.layout.Panel, pn.pane.Pane]]} -- A list of 'pages' to navigate
                between. The first page in pages is selected by default.
            page_outlet {pn.layout.ListPanel} -- The ListPanel to update when the user navigates to
                a new page
        """
        if css_classes:
            pnx.fontawesome.extend()
            menuitems = [
                NavigationButton(page, page_outlet=page_outlet, css_classes=css,)
                for page, css in zip(pages, css_classes,)
            ]
        else:
            menuitems = [NavigationButton(page=page, page_outlet=page_outlet,) for page in pages]

        # title = pnx.SubHeader(title, text_align=text_align)
        title = pn.layout.HSpacer(height=20)
        super().__init__(
            title, *menuitems, sizing_mode=sizing_mode, *args, **kwargs,
        )

        page_outlet.clear()
        page_outlet.append(pages[0])



"""Exposes Paths to the following assets

- BOOTSTRAP_PANEL_EXPRESS_CSS
- CODE_HILITE_PANEL_EXPRESS_CSS
- SCROLLBAR_PANEL_EXPRESS_CSS
- FONTAWESOME_PANEL_EXPRESS_CSS
"""
import pathlib

ROOT = pathlib.Path(__file__).parent
CSS_ROOT = ROOT / "css"
IMAGE_ROOT = ROOT / "images"

BOOTSTRAP_PANEL_EXPRESS_CSS = CSS_ROOT / "bootstrap_panel_express.css"
CODE_HILITE_PANEL_EXPRESS_CSS = CSS_ROOT / "code_hilite_panel_express.css"
SCROLLBAR_PANEL_EXPRESS_CSS = CSS_ROOT / "scrollbar_panel_express.css"
FONTAWESOME_PANEL_EXPRESS_CSS = CSS_ROOT / "fontawesome_panel_express.css"
DECK_GL_PANEL_EXPRESS_CSS = CSS_ROOT / "deck_gl_panel_express.css"



"""In this module we provide a list paths, urls and other usefull settings."""
GITHUB_URL = "https://github.com/MarcSkovMadsen/awesome-panel/"
GITHUB_BLOB_MASTER_URL = "https://github.com/MarcSkovMadsen/awesome-panel/blob/master/"
GITHUB_RAW_URL = "https://raw.githubusercontent.com/MarcSkovMadsen/awesome-panel/master/"
GITHUB_THUMBNAILS_URL = (
    "https://github.com/MarcSkovMadsen/awesome-panel/blob/master/assets/images/thumbnails/"
)
THUMBNAILS_ROOT = "assets/images/thumbnails/"



"""Functionality to speed up developing tests of Panel apps"""
from typing import Callable

from panel.layout import Column
from panel.pane import Markdown


class TestApp(Column):
    """Creates a Test App from the name and docstring of the test function"""

    __test__ = False  # We don't wan't pytest to collect this

    def __init__(self, test_func: Callable, *args, **kwargs):
        """## Creates a Test App from the name and docstring of the test function

        Displays
        - __name__
        - __doc__

        Has sizing_mode="stretch_width" unless otherwise specified

        Arguments:
            test_func {Callable} -- The function to create an app from.
        """
        text_str = test_func.__name__.replace("_", " ",).capitalize()
        text_str = "    # " + text_str

        if test_func.__doc__:
            if test_func.__doc__.startswith("    "):
                text_str += "\n\n" + test_func.__doc__
            else:
                text_str += "\n\n    " + test_func.__doc__

        text = Markdown(text_str)

        if "sizing_mode" not in kwargs and "width" not in kwargs and "height" not in kwargs:
            kwargs["sizing_mode"] = "stretch_width"

        super().__init__(text, *args, **kwargs)



"""In this module we define cards inspired by

- https://getbootstrap.com/docs/4.3/components/card/ and
- https://disjfa.github.io/bootstrap-tricks/card-collapse-tricks/
"""

from typing import List, Union

import panel as pn


class Card(pn.Column):
    """A Card inspired by the Bootstrap Card"""

    def __init__(
        self,
        header: str,
        body: Union[pn.viewable.Viewable, List[pn.viewable.Viewable],],
        collapsable: bool = False,
        **kwargs,
    ):
        if "css_classes" not in kwargs:
            kwargs["css_classes"] = []
        if "card" not in kwargs["css_classes"]:
            kwargs["css_classes"].append("card")
        if "sizing_mode" not in kwargs and "width" not in kwargs:
            kwargs["sizing_mode"] = "stretch_width"
        if not isinstance(body, list,):
            panels = [body]
        else:
            panels = body

        # Due to https://github.com/holoviz/panel/issues/903 we have to insert the content into a
        # column with relevant margin
        #
        content = self._get_card_content(panels)

        if not collapsable:
            header_pane = self.get_card_header(header)
            super().__init__(
                header_pane, content, **kwargs,
            )
            return

        collapse_button = pn.widgets.Button(
            name="-", width=30, sizing_mode="stretch_height", css_classes=["flat"],
        )

        def click_callback(event,):
            if event.new % 2 == 1:
                self.remove(content)
                collapse_button.name = "+"
            elif event.new > 0:
                self.append(content)
                collapse_button.name = "-"

        collapse_button.on_click(click_callback)
        header_row = pn.Row(
            f'<h5 class="card-header"">{header}</h5>',
            pn.layout.HSpacer(),
            collapse_button,
            css_classes=["card-header"],
        )
        super().__init__(
            header_row, content, **kwargs,
        )

    def clone(self, *objects, **params):
        # Hack. See https://github.com/holoviz/panel/issues/1060
        header, body = self.objects
        return super().clone(header.object, body, **params)

    def _get_card_content(self, panels: List[pn.viewable.Viewable],) -> pn.viewable.Viewable:
        """Combines the list of Viewables into a Viewable with the right css classes

        Args:
            panels (List[pn.viewable.Viewable]): A list of Viewables

        Returns:
            pn.viewable.Viewable: A Viewable of the input Viewables with the right css classes \
                applied.
        """
        # Due to https://github.com/holoviz/panel/issues/903 we have to insert the content into a
        # column with relevant margin
        #
        content = pn.Column(
            *[self.get_card_panel(panel) for panel in panels],
            css_classes=["card-body"],
            sizing_mode="stretch_width",
            margin=(0, 2, 2, 0,),
        )
        # Due to Bokeh formatting every card-panel is 5px inside the card-body
        # and thus we cannot get borders to overlap.
        # So the first panel should have no border to hide this fact
        if len(content) > 0:
            if content[0].css_classes and "card-panel" in content[0].css_classes:
                content[0].css_classes.remove("card-panel")
                content[0].css_classes.append("card-panel-first")
            else:
                content[0].css_classes = ["card-panel-first"]
        return content

    @staticmethod
    def get_card_header(text: str, **kwargs,) -> pn.pane.HTML:
        """[summary]

        Arguments:
            text {str} -- A header string like 'Card Header'. May also contain HTML tags like <a>
            and <i>

        Returns:
            pn.pane.HTML -- The header text as a HTML Pane
        """
        if "css_classes" not in kwargs:
            kwargs["css_classes"] = []
        if "card-header" not in kwargs["css_classes"]:
            kwargs["css_classes"].append("card-header")
        if "sizing_mode" not in kwargs and "width" not in kwargs:
            kwargs["sizing_mode"] = "stretch_width"
        if "margin" not in kwargs:
            kwargs["margin"] = 0
        object_ = f'<h5 class="card-header"">{text}</h5>'

        return pn.pane.HTML(object_, **kwargs,)

    @staticmethod
    def get_card_panel(obj, **kwargs,) -> pn.viewable.Viewable:
        """A Card Panel to be inserted into the body of the Card

        Arguments:
            obj {[type]} -- Any type that can be converted to a panel. An obj of type 'str' will
            always be converted to a HTML pane

        Returns:
            pn.viewable.Viewable -- A Viewable of the obj
        """
        if "css_classes" not in kwargs:
            kwargs["css_classes"] = []
        if "card-panel" not in kwargs["css_classes"]:
            kwargs["css_classes"].append("card-panel")
        if "sizing_mode" not in kwargs and "width" not in kwargs:
            kwargs["sizing_mode"] = "stretch_width"

        if isinstance(obj, str,):
            return pn.pane.Markdown(obj, **kwargs,)

        return pn.panel(obj, **kwargs,)



"""This module provides button to share on social media"""
import urllib.parse

import panel as pn
import param

DEFAULT_URL = "https://awesome-panel.org"
DEFAULT_CLASS = "fas fa-external-link-alt"
DEFAULT_TEXT = "Checkout"
FACEBOOK_CLASS = "fab fa-facebook-f"
LINKEDIN_CLASS = "fab fa-linkedin-in"
TWITTER_CLASS = "fab fa-twitter"
REDDIT_CLASS = "fab fa-reddit-alien"
MAIL_CLASS = "fas fa-envelope"


class ShareOnBase(param.Parameterized):
    """Base class for implementing ShareOnFacebook, ShareOnLinkedIn links etc.

    - The href property should be overridden
    """

    url = param.String(DEFAULT_URL)
    icon_class = param.String(DEFAULT_CLASS)
    text = param.String(DEFAULT_TEXT)

    @property
    def _url_parsed(self,):
        return urllib.parse.quote(self.url).replace("/", "%2F",)

    @property
    def href(self,) -> str:
        """The href to goto when clicked

        Override this method in a base class

        Raises:
            NotImplementedError:

        Returns:
            str: A href string
        """
        raise NotImplementedError()

    def __html__(self,) -> str:
        """A html string with link and icon tags

        Returns:
            str: A html string with link and icon tags
        """
        return (
            f'<a href="{self.href}" class="button-share-link">'
            f'<i class="{self.icon_class}"></i></a>'
        )

    @param.depends(
        "url", "icon_class",
    )
    def view(self,) -> pn.pane.HTML:
        """A HTML pane with the a link and icon

        Returns:
            pn.pane.HTML: A HTML pane with the link and icon
        """
        return pn.pane.HTML(self.__html__())


class ShareOnFacebook(ShareOnBase):
    """A Share on Facebook button"""

    icon_class = param.String(FACEBOOK_CLASS)

    @property
    def href(self,):
        return f"https://www.facebook.com/sharer/sharer.php?u={self._url_parsed}"


class ShareOnLinkedIn(ShareOnBase):
    """A Share on LinkedIn button"""

    icon_class = param.String(LINKEDIN_CLASS)

    @property
    def href(self,):
        return (
            f"http://www.linkedin.com/shareArticle?mini=true&url={self._url_parsed}"
            f"&title={self.text}"
        )


class ShareOnTwitter(ShareOnBase):
    """A Share on Twitter button"""

    icon_class = param.String(TWITTER_CLASS)

    @property
    def href(self,):
        return f"https://twitter.com/intent/tweet?url={self._url_parsed}&text={self.text}"


class ShareOnReddit(ShareOnBase):
    """A Share on Reddit button"""

    icon_class = param.String(REDDIT_CLASS)

    @property
    def href(self,):
        return f"https://reddit.com/submit?url={self._url_parsed}&amp;title={self.text}"


class ShareOnMail(ShareOnBase):
    """A Share on Mail button"""

    icon_class = param.String(MAIL_CLASS)

    @property
    def href(self,):
        return f"mailto:?subject={self._url_parsed}&amp;body={self.text}&nbsp;{self._url_parsed}"



"""Export of objects"""
from awesome_panel.express import fontawesome, spinners, templates, testing
from awesome_panel.express.bootstrap import ErrorAlert, InfoAlert, WarningAlert
from awesome_panel.express.pane._headings import Header, SubHeader, Title
from awesome_panel.express.pane.panes import Code
from awesome_panel.express.pane.pydeck import PyDeck
from awesome_panel.express.widgets.browserurl import BrowserUrlMixin
from awesome_panel.express.widgets.navigation import NavigationButton, NavigationMenu
from awesome_panel.express.widgets.progress_ext import ProgressExt

from .bootstrap.cards.cards import Card



"""## Font Awesome extension for Panel

As a user of Panel I would like to be able to use awesome layouts, formats and styles.

A part of this is being able to use [Font Awesome](https://fontawesome.com/) icons.

This module contains functionality for using Font Awesome as
[CSS Pseudo Elements](https://fontawesome.com/how-to-use/on-the-web/advanced/css-pseudo-elements)
"""
import urllib.request

import panel as pn

from awesome_panel.express.assets import FONTAWESOME_PANEL_EXPRESS_CSS

FONTAWESOME_CSS_URL = "https://use.fontawesome.com/releases/v5.11.2/css/all.css"

_EXTENDED = False

_FONTAWESOME_PANEL_EXPRESS_HEADER = """

div.bk.pas div.bk *::before, div.bk.pab div.bk *::before {
    display: inline-block;
    font-style: normal;
    font-variant: normal;
    text-rendering: auto;
    -webkit-font-smoothing: antialiased;
}
div.bk.pas div.bk *::before{
    font-family: "Font Awesome 5 Free";
}
div.bk.pab div.bk *::before {
    font-family:"Font Awesome 5 Brands"
}
"""


def extend():
    """## Extends Panel with functionality to use Font Awesome

    See [Font Awesome](https://fontawesome.com/) and
    [Font Awesome CSS Pseudo Elements]
    (https://fontawesome.com/how-to-use/on-the-web/advanced/css-pseudo-elements)
    for more information.

    To find icons refer to [Font Awesome Search]
    (https://fontawesome.com/icons/search).

    For examples refer to [W3C Font Awesome Intro]
    (https://www.w3schools.com/icons/fontawesome_icons_intro.asp)
    """
    global _EXTENDED  # pylint: disable=global-statement
    if not _EXTENDED:
        pn.config.raw_css.append(FONTAWESOME_PANEL_EXPRESS_CSS.read_text())
        pn.config.css_files.append(FONTAWESOME_CSS_URL)
        _EXTENDED = True


def get_fontawesome_panel_express() -> str:
    """Converts the official css file at FONTAWESOME_CSS_URL into it's panel
    representation

    Returns:
        str -- [description]
    """

    with urllib.request.urlopen(FONTAWESOME_CSS_URL) as file:
        fontawesome_css = file.read().decode("utf-8")

    return _to_fontawesome_panel_express(fontawesome_css)


def _to_fontawesome_panel_express(font_awesome_css: str,) -> str:
    """Converts a css string like FONTAWESOME_CSS_URL into it's panel
    representation

    Returns:
        str -- [description]
    """
    css = font_awesome_css.replace("}.", "}\n.",)
    lines_in = css.split("\n")
    lines_out = []
    for line in lines_in:
        if ":before{" in line:
            line = line.replace(".fa-", "div.bk.pa-",)
            line = line.replace(":before{", " div.bk *::before{",)
            lines_out.append(line)
    return _FONTAWESOME_PANEL_EXPRESS_HEADER + "\n".join(lines_out)



"""This module provides services related to Resources"""
from collections import defaultdict
from typing import Dict, List, Optional

from awesome_panel.database.resources import RESOURCES
from awesome_panel.shared.models import Author, Resource, Tag


def filter_by_tags(resources: List[Resource], tags: List[Tag],) -> List[Resource]:
    """The resources having all of the specified Tags

    If tags is the empty list all resources are returned

    Arguments:
        resources {List[Resource]} -- A list of Resources
        tags {List[Tag]} -- A list of Tags

    Returns:
        [List[Resource]] -- A list of Resources
    """
    if tags:
        resources_ = []
        for resource in resources:
            if set(tags).issubset(resource.tags):
                resources_.append(resource)
        return resources_

    return resources


def filter_by_is_awesome(resources: List[Resource],) -> List[Resource]:
    """The resources being that is_awesome

    Arguments:
        resources {List[Resource]} -- A list of resources

    Returns:
        List[Resource] - A list of awesome resources
    """
    return [resource for resource in resources if resource.is_awesome]


def filter_by_author(resources: List[Resource], author: Author,) -> List[Resource]:
    """The resources by the specified author

    Arguments:
        resources {List[Resource]} -- A list of resources

    Returns:
        List[Resource] - A list of resources by the specified author
    """
    return [resource for resource in resources if resource.author == author]


def sort_resources(resources: List[Resource],) -> List[Resource]:
    """The list of resources sorted by name

    Returns:
        [List[Resource]] -- The list of resources sorted by name
    """
    return sorted(resources, key=lambda x: x.name,)


def get_resources(
    tags: List[Tag], author: Optional[Author] = None, awesome_resources_only: bool = True,
) -> List[Resource]:
    """A list of resources

    Arguments:
        tags {List[Tag]} -- If non-empty then the list of Resources is reduced to
        Resources having one of the specified tags

    Keyword Arguments:
        author {Author} -- If an author is specified the list of resources is reduced to
        resources by that author (default: {None})
        awesome_resources_only {bool} -- If True then the list is reduced to
        Resources with is_awesome equal to True(default: {True})

    Returns:
        List[Resource] -- A list of Resources
    """
    resources = RESOURCES
    if author:
        resources = filter_by_author(resources, author,)
    if awesome_resources_only:
        resources = filter_by_is_awesome(resources)
    resources = sort_resources(resources)
    resources = filter_by_tags(resources, tags,)
    return resources


def to_markdown(resources: List[Resource], report_by_tag: bool = True,) -> str:
    """Converts the specified resources to MarkDown

    Arguments:
        resources {List[Resource]} -- [description]

    Optional Arguments:
        report_by_tag {bool} - If True the text is split into sections by Tags

    Returns:
        [str] -- Bulleted Markdown List of Resources
    """

    if report_by_tag:
        markdown_bullets = []
        resources_dict: Dict[Tag, List[Resource],] = defaultdict(list)
        for resource in resources:
            resources_dict[resource.tags[0]].append(resource)

        for tag in sorted(resources_dict.keys(), key=lambda x: x.name,):
            markdown_bullets.append(f"\n### {tag.name}\n")
            for resource in resources_dict[tag]:
                markdown_bullets.append(resource.to_markdown_bullet())
    else:
        markdown_bullets = [resource.to_markdown_bullet() for resource in resources]

    markdown = "\n".join(markdown_bullets)

    return markdown


def get_resources_markdown(
    tags: List[Tag], author: Optional[Author] = None, awesome_resources_only: bool = True,
) -> str:
    """A bulleted Markdown list of resources filtered as specified

    Arguments:
        tags {List[Tag]} -- A list of tags to filter to. If the list is empty [] then we
do no filtering on Tags

    Keyword Arguments:
        Author {Author} -- An author to filter to. If author is None
then we do no filtering on author. (default: {None})
        awesome_resources_only {bool} -- [description] (default: {True})

    Returns:
        str -- A bulleted Markdown list of resources filtered as specified
    """
    resources = get_resources(tags, author, awesome_resources_only,)
    return to_markdown(resources, not tags,)



"""This module contains functionality to keep a param.Parametrized class
in sync with the browser url


"""
import urllib

import panel as pn


class BrowserUrlMixin:  # pylint: disable=too-few-public-methods
    """This Mixin enables parameters from a param.Parametrized Class to be shown in the url or the
    browser. For example like http://localhost/example_app?country=Norway

    Mix this class with the param.Parametrized class and add the set_browser_url_parameters HTML
    pane to your app in order to get url updates

    Example use case

    class _Country(pnx.BrowserUrlMixin, param.Parameterized):
        country = param.String()

        @param.depends("country")
        def set_browser_url_parameters(self):
            return super().set_browser_url_parameters()
    """

    def __init__(
        self, *args, **kwargs,
    ):
        super(BrowserUrlMixin, self,).__init__(
            *args, **kwargs,
        )
        self._get_browser_url_parameters()

    def _get_browser_url_parameters(self,):
        """Sets the parameters from the browser url parameters"""
        for (key, value,) in pn.state.session_args.items():
            if key in self._parameter_dict():
                value_str = value[0].decode("utf8")
                self.set_param(
                    key, value_str,
                )

    def set_browser_url_parameters(self,) -> pn.pane.HTML:
        """A HTML Pane. Should be included in the app in order
        to update the browser url when a parameter changes.

        Returns:
            pn.pane.HTML -- A pane containing the javascript script to update the browser url
        """
        return pn.pane.HTML(self._browser_url_parameters_script())

    def _browser_url_parameters_script(self,) -> str:
        if len(self.get_param_values()) > 1:  # type: ignore
            state = f'{{param: "{self._urlencode()}"}}'
            title = ""
            url = f"?{self._urlencode()}"

            return f"""<script>window.history.replaceState({state},"{title}","{url}");</script>"""
        return ""

    def _parameter_dict(self,):
        return {item[0]: item[1] for item in self.get_param_values() if item[0] != "name"}

    def _urlencode(self,):
        return urllib.parse.urlencode(self._parameter_dict())



"""This module provides the EmptyComponent to be used if no component is provided to the
Designer"""
import panel as pn
import param


class EmptyComponent(param.Parameterized):
    """Dummy Component used by the Designer when no component is provided"""

    view = param.ClassSelector(class_=pn.Column)

    def __init__(self, **params):
        super().__init__(**params)

        self.view = pn.Column("# Empty Component")



"""In this module we implement a title, heading and subheading"""
from typing import Optional

import panel as pn


class HeadingBase(pn.pane.Markdown):
    """Base class for headings like Title, Header and SubHeader"""

    def __init__(  # pylint: disable=too-many-arguments
        self,
        text: str = "",
        level: int = 1,
        sizing_mode="stretch_width",
        text_align="left",
        url: Optional[str] = None,
        **kwargs,
    ):
        """Base class for headings like Title, Header and SubHeader

        Keyword Arguments:
            text {str} -- The heading to display (default: {""})
            level {int} -- 1 is Title, 2 is Header, ... (default: {1})
            sizing_mode {str} -- (default: {"stretch_width"})
            text_align {str} -- The header can be aligned 'left', 'center' or 'right'
                (default: {"left"})
            url {Optional[str]} -- If not None the heading will link to the url
        """
        if url:
            heading = f"{'#'*level} [{text}]({url})"
        else:
            heading = f"{'#'*level} {text}"
        if "style" in kwargs:
            kwargs["style"]["text-align"] = text_align
        else:
            kwargs["style"] = {"text-align": text_align}
        super().__init__(
            heading, sizing_mode=sizing_mode, **kwargs,
        )


class Title(HeadingBase):
    """A Markdown Pane with the title"""

    def __init__(
        self,
        text: str = "",
        sizing_mode="stretch_width",
        text_align="left",
        url: Optional[str] = None,
        **kwargs,
    ):
        """A Markdown Pane with the title

        Keyword Arguments:
            text {str} -- The title to display (default: {""})
            sizing_mode {str} -- (default: {"stretch_width"})
            text_align {str} -- The header can be aligned 'left', 'center' or 'right'
                (default: {"left"})
            url {Optional[str]} -- If not None the heading will link to the url
        """
        super().__init__(
            text=text, level=1, sizing_mode=sizing_mode, text_align=text_align, url=url, **kwargs,
        )


class Header(HeadingBase):
    """A Markdown Pane with the header"""

    def __init__(
        self,
        text: str = "",
        sizing_mode="stretch_width",
        text_align="left",
        url: Optional[str] = None,
        **kwargs,
    ):
        """A Markdown Pane with the header

        Keyword Arguments:
            text {str} -- The header to display (default: {""})
            sizing_mode {str} -- (default: {"stretch_width"})
            text_align {str} -- The header can be aligned 'left', 'center' or 'right'
                (default: {"left"})
            url {Optional[str]} -- If not None the heading will link to the url
        """
        super().__init__(
            text=text, level=2, sizing_mode=sizing_mode, text_align=text_align, url=url, **kwargs,
        )


class SubHeader(HeadingBase):
    """A Markdown Pane with the sub header"""

    def __init__(
        self,
        text: str = "",
        sizing_mode="stretch_width",
        text_align="left",
        url: Optional[str] = None,
        **kwargs,
    ):
        """A Markdown Pane with the sub header

        Keyword Arguments:
            text {str} -- The sub header to display (default: {""})
            sizing_mode {str} -- (default: {"stretch_width"})
            text_align {str} -- The header can be aligned 'left', 'center' or 'right'
                (default: {"left"})
            url {Optional[str]} -- If not None the heading will link to the url
        """
        super().__init__(
            text=text, level=3, sizing_mode=sizing_mode, text_align=text_align, url=url, **kwargs,
        )



"""Provides access to Bootstrap inspired functionality like `ErrorAlert`, `InfoAlert`,
`WarningAlert`, `Card` and the `extend` function that needs to be run before you start using the
Bootstrap functionality.
"""
from .bootstrap import ErrorAlert, InfoAlert, WarningAlert, extend
from .cards.cards import Card



"""Implementation of a PyDeck pane"""
# Inspired by PyDeck
# Jinja: https://github.com/uber/deck.gl/blob/master/bindings/pydeck/pydeck/io/templates/index.j2

import panel as pn
import param
import pydeck as pdk

from awesome_panel.express.assets import DECK_GL_PANEL_EXPRESS_CSS

_EXTENDED = False


DECK_GL_PANEL_EXPRESS_JS = (
    "https://cdn.jsdelivr.net/npm/@deck.gl/jupyter-widget@^8.0.0/dist/index.js"
)
MAPBOX_GL_JS = "https://api.tiles.mapbox.com/mapbox-gl-js/v0.50.0/mapbox-gl.js"

# Right now this is not implemented as a Pane
# But eventually it should be
class PyDeck(param.Parameterized):
    """A PyDeck Pane

    This Pane can be used to convert a PyDeck object to a Panel in Panel.
    """

    object = param.Parameter(
        default=None,
        doc="""
        The pydeck Deck object being wrapped.""",
    )

    def __init__(
        self, object: pdk.Deck = None, **params,  # pylint: disable=redefined-builtin
    ):
        self.object = object
        super().__init__(**params)

    def _get_js(
        self, container: str = "deck-container",
    ):
        # Get
        json_input = self.object.to_json()
        mapbox_key = self.object.mapbox_key
        tooltip = self.object.deck_widget.tooltip

        # Clean
        if isinstance(json_input, str,):
            json_input = json_input.replace('"', "'",)
        if not isinstance(mapbox_key, str,):
            mapbox_key = ""
        if isinstance(tooltip, bool,):
            tooltip = str(tooltip).lower()

        return f"""\
const jsonInput = {json_input};
const MAPBOX_API_KEY = '{mapbox_key}';
const tooltip = {tooltip};

const deck = createDeck({{
    mapboxApiKey: MAPBOX_API_KEY,
    container: document.getElementById('{container}'),
    jsonInput,
    tooltip
}});"""

    @param.depends("object")
    def _repr_html_(self,):
        # Inspired by the implementation of pydeck.Deck.to_html
        # See https://github.com/uber/deck.gl/blob/master/bindings/pydeck/pydeck/bindings/deck.py
        if not self.object:
            return ""

        js_code = self._get_js()

        html = f"""\
<div id="deck-container"></div>
<script>
{js_code}
</script>"""
        return html

    @property
    def as_pane(self,) -> pn.pane.HTML:
        """Converts the _repr_html_ to a HTML pane"""
        return pn.pane.HTML(self._repr_html_())

    @staticmethod
    def extend():
        """## Extends Panel with CSS and JS functionality to use PyDeck and Deck.Gl."""
        global _EXTENDED  # pylint: disable=global-statement
        if not _EXTENDED:
            pn.config.raw_css.append(DECK_GL_PANEL_EXPRESS_CSS.read_text())
            pn.config.js_files["deck"] = DECK_GL_PANEL_EXPRESS_JS
            pn.config.js_files["mapbox"] = MAPBOX_GL_JS
            _EXTENDED = True



"""Functionality for awesome-streamlit.org"""
from panel.pane import Markdown


def title_awesome(body: str,) -> Markdown:
    """An *Awesome Panel* title as a Markdown with

    - the text like 'Awesome Panel About'
    - the [Awesome Badge](https://cdn.rawgit.com/sindresorhus/awesome/\
d7305f38d29fed78fa85652e3a63e154dd8e8829/media/badge.svg)

    Arguments:
        body (str): Some title like 'About'

    Returns:
        Markdown: An 'Awesome Panel {body} title with a link and the awesome badge.
    """
    return Markdown(
        f"# Awesome Panel {body} "
        "![Awesome Badge](https://cdn.rawgit.com/sindresorhus/awesome/"
        "d7305f38d29fed78fa85652e3a63e154dd8e8829/media/badge.svg)"
    )



"""This module implements the ReloadService. The ReloadService is used by the Designer.
For each component you want access to in the Designer you should provide a seperate
Reload Service"""

import datetime
import importlib
import pathlib
import sys
import traceback

import param


class ReloadService(param.Parameterized):  # pylint: disable=too-many-instance-attributes
    """The ReloadService is used by the Designer.
For each component you want access to in the Designer you should provide a seperate
Reload Service

Args:
    component ([type]): For now the components that are know to be supported are

    - subclasses of `pn.layout.Reactive`
    - subclasses of `param.Parameterized` with a `view` parameter which is a subclass of
    `pn.layout.Reactive`

Please NOTE that in order for the reload service to be able to reload the compoonent, the component
specified cannot be defined in the __main__ file.

Example
-------

```python
TITLE_COMPONENT = ReloadService(
    component=components.TitleComponent, css_path=COMPONENT_CSS, js_path=COMPONENT_JS,
)
EMPTY_COMPONENT = ReloadService(
    component=components.EmptyComponent, css_path=COMPONENT_CSS, js_path=COMPONENT2_JS,
)
```"""

    component = param.Parameter(allow_None=False)
    component_parameters = param.Dict()
    component_instance = param.Parameter()
    css_path = param.Parameter(constant=True)
    js_path = param.Parameter(constant=True)
    modules_to_reload = param.List()

    reload_component = param.Action(label="RELOAD COMPONENT")
    reload_css_file = param.Action(label="RELOAD CSS")
    reload_js_file = param.Action(label="RELOAD JS")

    css_text = param.String()
    js_text = param.String()

    reloading = param.Boolean(default=False)
    last_reload = param.String(constant=True)
    error_message = param.String()

    def __init__(self, component, **params):
        if not isinstance(params, dict):
            params = {}
        params["component"] = component
        super().__init__(**params)

        with param.edit_constant(self):
            self.name = self.component.name

        self.reload_component = self._reload_component
        self.reload_css_file = self._reload_css_file
        self.reload_js_file = self._reload_js_file

    def __repr__(self):
        return f"ReloadService({self.name})"

    def __str__(self):
        return f"ReloadService({self.name})"

    def _reload_component(self, _=None):
        try:
            self._signal_reload_start()

            if self.component_instance is not None:
                for mod in self.modules_to_reload:  # pylint: disable=not-an-iterable
                    importlib.reload(mod)

                mod = sys.modules[self.component.__module__]
                importlib.reload(mod)
                with param.edit_constant(self):
                    self.component = getattr(mod, self.component.__name__)

            if self.component_parameters:
                # pylint: disable=not-a-mapping
                self.component_instance = self.component(**self.component_parameters)
            else:
                self.component_instance = self.component()

            self._reset_error_message()
        except Exception as ex:  # pylint: disable=broad-except
            self._report_exception(ex)
        finally:
            self._signal_reload_end()

    def _reload_css_file(self, _=None):
        try:
            self._signal_reload_start()
            if not self.css_path:
                pass
            elif isinstance(self.css_path, pathlib.Path):
                self.css_text = self.css_path.read_text()
            else:
                raise NotImplementedError

            self._reset_error_message()
        except Exception as ex:  # pylint: disable=broad-except
            self._report_exception(ex)
        finally:
            self._signal_reload_end()

    def _reload_js_file(self, _=None):
        try:
            self._signal_reload_start()
            if not self.js_path:
                pass
            elif isinstance(self.js_path, pathlib.Path):
                self.js_text = self.js_path.read_text()
            else:
                raise NotImplementedError

            self._reset_error_message()
        except Exception as ex:  # pylint: disable=broad-except
            self._report_exception(ex)
        finally:
            self._signal_reload_end()

    def _signal_reload_start(self):
        self.reloading = True
        print("reload start", self.name, datetime.datetime.now())

    def _report_exception(self, ex):  # pylint: disable=unused-argument
        self.error_message = "# Error " + traceback.format_exc()
        print(self.name, self.error_message)

    def _signal_reload_end(self):
        self.reloading = False
        with param.edit_constant(self):
            self.last_reload = str(datetime.datetime.now())
        print("reload end", self.name, datetime.datetime.now())

    def _reset_error_message(self):
        self.error_message = ""



"""In this module we provide the functionality of a Modal.

The Modal can be used to focus some kind of information like text, images, chart or an interactive
dashboard.

The implementation is inspired by

- https://css-tricks.com/considerations-styling-modal/
    - https://codepen.io/henchmen/embed/preview/PzQpvk
- https://getbootstrap.com/docs/4.3/components/modal/
"""
import panel as pn
import param

_CSS = """
.bk.modal {
  /* This way it could be display flex or grid or whatever also. */
  display: block;
  max-width: 100%;
  max-height: 100%;
  position: fixed!important;
  z-index: 100;

  left: 0!important;
  top: 0!important;
  bottom: 0!important;
  right: 0!important;

  margin: auto!important;
  box-shadow: 5px 5px 20px grey;
  box-shadow: 0 0 60px 10px rgba(0, 0, 0, 0.9);

  border: 1px solid rgba(0,0,0,.125);
  border-radius: 0.25rem;
}
.closed {
  display: none!important;
}
.modal-overlay {
  position: fixed;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
  z-index: 50;

  background: rgba(0, 0, 0, 0.6);
}
.modal-body {
  overflow: auto;
}
"""


class Modal(param.Parameterized):
    """The Modal can be used to focus some kind of information like text, images, chart or an
    interactive dashboard.

    In order to use this modal you need to

    - Instantiate the Modal
    - Add the CSS from the get_css function to the app
        - using `pn.config.raw_css.append` or
        - directly in your template

    The implementation is inspired by

    - https://css-tricks.com/considerations-styling-modal/
        - https://codepen.io/henchmen/embed/preview/PzQpvk
    - https://getbootstrap.com/docs/4.3/components/modal/
    """

    title = param.String(default="Modal")
    body = param.List()

    def __init__(self, **params):
        super().__init__(**params)
        self.modal_overlay = pn.pane.HTML('<div class="modal-overlay" id="modal-overlay"></div>')
        self.close_button = pn.widgets.Button(
            name="X", css_classes=["close-modal-button"], width=50,
        )
        self.close_button.js_on_click(
            code="""
    var modal = document.querySelector(".bk.modal");
    var modalOverlay = document.querySelector("#modal-overlay");
    modal.classList.toggle("closed");
    modalOverlay.classList.toggle("closed");
    """
        )
        self._modal_title = pn.pane.Markdown("# " + self.title)
        self._modal_body = pn.Column(*self.body)  # pylint: disable=not-an-iterable

        self.modal = pn.Column(
            pn.Column(
                pn.Row(self._modal_title, pn.layout.HSpacer(), self.close_button,),
                pn.layout.Divider(),
                self._modal_body,
                sizing_mode="stretch_width",
                margin=10,
            ),
            background="white",
            width=400,
            height=400,
            css_classes=["modal"],
        )

    @staticmethod
    def get_open_modal_button(name: str = "Open Modal", **kwargs) -> pn.widgets.Button:
        """A Button to open the modal with"""
        open_modal_button = pn.widgets.Button(
            name=name, css_classes=["open-modal-button"], **kwargs
        )
        open_modal_button.js_on_click(
            code="""
    var modal = document.querySelector(".modal");
    var modalOverlay = document.querySelector("#modal-overlay");

    modal.classList.toggle("closed");
    modalOverlay.classList.toggle("closed");
    """
        )

        return open_modal_button

    @staticmethod
    def get_css() -> str:
        """Add the CSS from this function to the app

        - using `pn.config.raw_css.append` or
        - directly in your template

        Returns:
            str: The css string
        """
        return _CSS

    @param.depends(
        "title", watch=True,
    )
    def set_modal_title(self,):
        """Updates the title of the modal"""
        self._modal_title.object = "# " + self.title

    @param.depends(
        "body", watch=True,
    )
    def set_modal_body(self,):
        """Updates the body of the modal"""
        self._modal_body[:] = self.body



"""Configuration and settings for the Designer. TO BE REFACTORED."""
ACTION_PARAMETERS = [
    "reload_component",
    "reload_css_file",
    "reload_js_file",
    "last_reload",
]

CSS = """
body {
    margin: 0px;
    max-width: 100vw;
    min-height: 100vh;
    margin: 0px;
}

.bk.designer-design-pane a:link {
    color: black;
    font-style: normal;
    text-decoration: none;
}

.bk-root .bk-btn-primary, .bk-root .bk-btn-primary:hover, .bk-root .bk-btn-primary.bk-active {
    background: #66bb6a;
    border-color: #66bb6a;
}

.bk.designer-design-pane {
    color: black;
    border-left-color: #9E9E9E;
    border-left-width: 1px;
    border-left-style: solid;
    border-bottom-color: #9E9E9E;
    border-bottom-width: 1px;
    border-bottom-style: solid;
}

.bk.designer-centered-component {
    color: black;
    border-right-color: #9E9E9E;
    border-right-width: 1px;
    border-right-style: solid;
    border-bottom-color: #9E9E9E;
    border-bottom-width: 1px;
    border-bottom-style: solid;
    border-left-color: #9E9E9E;
    border-left-width: 1px;
    border-left-style: solid;
    border-top-color: #9E9E9E;
    border-top-width: 1px;
    border-top-style: solid;
}
"""



"""Imports to be exposed to the user of the package are listed here"""
from awesome_panel.database.authors import AUTHORS
from awesome_panel.database.resources import RESOURCES
from awesome_panel.database.tags import TAGS



"""In this module we define all apps in the gallery as Resources and expose them via the
**APPS_IN_GALLERY** list.

Please note that all apps in the list

- be located in https://raw.githubusercontent.com/MarcSkovMadsen/awesome-panel/master/gallery/
- include the tags *tags.CODE* and *tags.APP_IN_GALLERY* as a minimum
"""
from awesome_panel.database import authors, tags
from awesome_panel.database.settings import GITHUB_BLOB_MASTER_URL, THUMBNAILS_ROOT
from awesome_panel.shared.models import Resource

GITHUB_GALLERY_URL = GITHUB_BLOB_MASTER_URL + "src/pages/gallery/"

# Please keep this list sorted by name
APPS_IN_GALLERY = [
    Resource(
        name="Bootstrap Dashboard",
        url=GITHUB_GALLERY_URL + "bootstrap_dashboard/main.py",
        thumbnail_path=THUMBNAILS_ROOT + "bootstrap_dashboard.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Custom Bokeh Model",
        url=GITHUB_GALLERY_URL + "custom_bokeh_model/custom.py",
        thumbnail_path=THUMBNAILS_ROOT + "custom_bokeh_model.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="DataExplorer - Loading...",
        url=GITHUB_GALLERY_URL + "dataexplorer_loading/dataexplorer_loading.py",
        thumbnail_path=THUMBNAILS_ROOT + "dataexplorer_loading.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Image Classifier",
        url=GITHUB_GALLERY_URL + "image_classifier/image_classifier.py",
        thumbnail_path=THUMBNAILS_ROOT + "image_classifier.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Owid Choropleth Map",
        url=GITHUB_GALLERY_URL + "owid_choropleth_map/main.py",
        thumbnail_path=THUMBNAILS_ROOT + "owid_choropleth_map.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Kickstarter Dashboard",
        url=GITHUB_GALLERY_URL + "kickstarter_dashboard/main.py",
        thumbnail_path=THUMBNAILS_ROOT + "kickstarter_dashboard.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Param Reference Example",
        url=GITHUB_GALLERY_URL + "param_reference_example/param_reference_example.py",
        thumbnail_path=THUMBNAILS_ROOT + "param_reference_example.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Yahoo Query",
        url=GITHUB_GALLERY_URL + "yahooquery_app/yahooquery_app.py",
        thumbnail_path=THUMBNAILS_ROOT + "yahooquery_app.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Test Bootstrap Alerts",
        url=GITHUB_GALLERY_URL + "awesome_panel_express_tests/test_bootstrap_alerts.py",
        thumbnail_path=THUMBNAILS_ROOT + "test_bootstrap_alerts.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY, tags.AWESOMEPANEL_EXPRESS,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Test Bootstrap Card",
        url=GITHUB_GALLERY_URL + "awesome_panel_express_tests/test_bootstrap_card.py",
        thumbnail_path=THUMBNAILS_ROOT + "test_bootstrap_card.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY, tags.AWESOMEPANEL_EXPRESS,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Test Code",
        url=GITHUB_GALLERY_URL + "awesome_panel_express_tests/test_code.py",
        thumbnail_path=THUMBNAILS_ROOT + "test_code.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY, tags.AWESOMEPANEL_EXPRESS,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Test DataFrame",
        url=GITHUB_GALLERY_URL + "awesome_panel_express_tests/test_dataframe.py",
        thumbnail_path=THUMBNAILS_ROOT + "test_dataframe.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY, tags.AWESOMEPANEL_EXPRESS,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Test Divider",
        url=GITHUB_GALLERY_URL + "awesome_panel_express_tests/test_divider.py",
        thumbnail_path=THUMBNAILS_ROOT + "test_divider.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY, tags.AWESOMEPANEL_EXPRESS,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Test FontAwesome",
        url=GITHUB_GALLERY_URL + "awesome_panel_express_tests/test_fontawesome.py",
        thumbnail_path=THUMBNAILS_ROOT + "test_fontawesome.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY, tags.AWESOMEPANEL_EXPRESS,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Test Headings",
        url=GITHUB_GALLERY_URL + "awesome_panel_express_tests/test_headings.py",
        thumbnail_path=THUMBNAILS_ROOT + "test_headings.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY, tags.AWESOMEPANEL_EXPRESS,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Test Markdown",
        url=GITHUB_GALLERY_URL + "awesome_panel_express_tests/test_markdown.py",
        thumbnail_path=THUMBNAILS_ROOT + "test_markdown.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY, tags.AWESOMEPANEL_EXPRESS,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Test Modal",
        url=GITHUB_GALLERY_URL + "awesome_panel_express_tests/test_modal.py",
        thumbnail_path=THUMBNAILS_ROOT + "test_modal.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY, tags.AWESOMEPANEL_EXPRESS,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Test Progress Extension",
        url=GITHUB_GALLERY_URL + "awesome_panel_express_tests/test_progress_ext.py",
        thumbnail_path=THUMBNAILS_ROOT + "test_progress_ext.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY, tags.AWESOMEPANEL_EXPRESS,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Test Share Links",
        url=GITHUB_GALLERY_URL + "awesome_panel_express_tests/test_share_links.py",
        thumbnail_path=THUMBNAILS_ROOT + "test_share_links.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY, tags.AWESOMEPANEL_EXPRESS,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Test Spinners",
        url=GITHUB_GALLERY_URL + "awesome_panel_express_tests/test_spinners.py",
        thumbnail_path=THUMBNAILS_ROOT + "test_spinners.png",
        tags=[tags.CODE, tags.APP_IN_GALLERY, tags.AWESOMEPANEL_EXPRESS,],
        is_awesome=True,
        author=authors.MARC_SKOV_MADSEN,
    ),
]



"""This module implements the CenteredComponent which can be used to center a component


"""
import panel as pn
import param


class CenteredComponent(pn.Column):
    """The CenteredComponent can be used to center a component"""

    component = param.Parameter()

    def __init__(self, component=None, **params):
        self._rename["component"] = None

        if "sizing_mode" not in params:
            params["sizing_mode"] = "stretch_both"

        super().__init__(**params)
        self.component = component

        self._spacer = pn.Spacer()
        self._vspacer = pn.layout.HSpacer(height=20)
        self._update()

    @param.depends("component", watch=True)
    def _update(self):
        if self.component is None:
            self[:] = [self._spacer]
            return

        component = self.component
        if isinstance(component, pn.layout.Reactive):
            main_content = component
        elif hasattr(component, "view") and component.view:
            main_content = component.view
        else:
            raise NotImplementedError

        main_content.align = "center"
        if main_content.css_classes is None:
            main_content.css_classes = []
        if "designer-centered-component" not in main_content.css_classes:
            main_content.css_classes.append("designer-centered-component")
        main_content.margin = (10, 25, 10, 5)
        self[:] = [
            main_content,
            self._vspacer,
        ]

    def __repr__(self, depth=0, max_depth=0):
        return f"PanelDesigner({self.name})"

    def __str__(self):
        return f"PanelDesigner({self.name})"



"""This module implements the StoppedComponent. The view of the StoppedComponent is shown to the
user when he/ she exits the Designer"""
import panel as pn
import param

MESSAGE = """\
# Stopped

Thanks for using the Panel Designer by awesome-panel.org
"""


class StoppedComponent(param.Parameterized):
    """The viewStoppedComponent is shown to the user when he/ she exits the Designer"""

    view = param.ClassSelector(class_=pn.pane.Markdown)

    def __init__(self, **params):
        super().__init__(**params)

        self.view = pn.pane.Markdown(MESSAGE)



"""Contains spinner functionality"""
import panel as pn

DEFAULT_URL = (
    "https://github.com/MarcSkovMadsen/awesome-panel/blob/master/"
    "package/awesome_panel/express/assets/images/spinner.gif?raw=true"
)
FACEBOOK_URL = (
    "https://github.com/MarcSkovMadsen/awesome-panel/blob/master/"
    "package/awesome_panel/express/assets/images/spinner_facebook.gif?raw=true"
)
WIDTH = 64  # Hack: Such that bokeh layout engine knows image size and can layout it correctly


class SpinnerBase(pn.pane.HTML):
    """The base spinner class. To be inherited from. DON'T USE THIS ONE"""

    url = DEFAULT_URL
    width = 64
    height = 64

    def __init__(
        self, *args, **kwargs,
    ):
        """A Spinner"""
        text = f'<img src="{self.url}"></img>'
        super().__init__(
            text, width=self.width, height=self.height, *args, **kwargs,
        )

    def center(self,) -> pn.Column:
        """Places the spinner in the center of a responsive Column

        Returns:
            pn.Column -- A responsive column with a spinner]
        """
        return pn.Column(
            pn.layout.VSpacer(),
            pn.Row(pn.layout.HSpacer(), self, pn.layout.HSpacer(),),
            pn.layout.VSpacer(),
            sizing_mode="stretch_both",
        )


class DefaultSpinner(SpinnerBase):
    """The Default spinner"""

    url = DEFAULT_URL


class FacebookSpinner(SpinnerBase):
    """The Facebook spinner"""

    url = FACEBOOK_URL



"""Helper Services"""
import inspect
import pathlib
from types import ModuleType

from awesome_panel.database.settings import GITHUB_BLOB_MASTER_URL

ROOT_PATH = str(pathlib.Path.cwd()).lower()


def module_to_github_url(module: ModuleType,) -> str:
    """## The link to the GitHub Source File of the URL

    Arguments:
        module {ModuleType} -- A module, for example package.awesome_panel.app.services
    Raises:
        ValueError: If the file path of the module is not in the project
    Returns:
        str -- A link to the GitHub file, for example
        'https://github.com/marcskovmadsen/awesome-panel/package/awesome_panel/app/services.py'
    """
    file_absolute = inspect.getfile(module).lower()
    if not file_absolute.startswith(ROOT_PATH):
        raise ValueError("Module is not in project!")
    file_relative = file_absolute.replace(ROOT_PATH, "",)
    file_relative = file_relative[1:].replace("\\", "/",)
    return GITHUB_BLOB_MASTER_URL + file_relative



"""Bootstrap inspired functionality"""
import panel as pn
from panel.pane import Markdown

from awesome_panel.express.assets import BOOTSTRAP_PANEL_EXPRESS_CSS

BOOTSTRAP_CSS_URL = "https://maxcdn.bootstrapcdn.com/bootstrap/4.4.1/css/bootstrap.min.css"

_EXTENDED = False


def extend():
    """## Extends Panel with functionality to use Bootstrap Inspired components and styles

    See [Get Bootstrap](https://getbootstrap.com/).

    For examples see also [W3C Bootstrap Tutorial]
    (https://www.w3schools.com/bootstrap4/default.asp)

    - We don't use the bootstrap javascript like jquery, popper and bootstrap because it does not
    play well with the Bookeh Layout Engine
    """
    global _EXTENDED  # pylint: disable=global-statement
    if not _EXTENDED:
        pn.config.raw_css.append(BOOTSTRAP_PANEL_EXPRESS_CSS.read_text())
        # pn.config.css_files.append(BOOTSTRAP_CSS_URL)
        _EXTENDED = True


class InfoAlert(Markdown):
    """

    An Info Alert that renders Markdown

    CSS Styling can be done via the classes 'alert' and 'alert-info'. See the raw_css attribute

    Don't set sizing_mode="stretch_width" as this will result in problems.
    See https://github.com/holoviz/panel/issues/829
    """

    def __init__(
        self, text: str, **kwargs,
    ):
        """An Info Alert that renders Markdown

        CSS Styling can be done via the classes 'alert' and 'alert-info'

        Arguments:
            text {str} -- Some MarkDown text

        """
        if "css_classes" in kwargs:
            if not "alert" in kwargs["css_classes"]:
                kwargs["css_classes"].append("alert")
            if not "alert-info" in kwargs["css_classes"]:
                kwargs["css_classes"].append("alert-info")
        else:
            kwargs["css_classes"] = [
                "alert",
                "alert-info",
            ]
        if "margin" not in kwargs:
            kwargs["margin"] = (0, 0, 25, 0)
        if "sizing_mode" not in kwargs:
            kwargs["sizing_mode"] = "stretch_width"

        super().__init__(
            text, **kwargs,
        )

    raw_css = """
.bk.alert {
    position: relative;
    padding: 0.75rem 1.25rem;
    border: 1px solid transparent;
    border-radius: 0.25rem;
}

.bk.alert-info {
    color: #0c5460;
    background-color: #d1ecf1;
    border-color: #bee5eb;
}

.bk.alert-info hr {
    border-top-color: #abdde5;
}
"""


class WarningAlert(pn.pane.Markdown):
    """An Warning Alert that renders Markdown

    CSS Styling can be done via the classes 'alert' and 'alert-warning'. See the raw_css attribute

    Don't set sizing_mode="stretch_width" as this will result in problems.
    See https://github.com/holoviz/panel/issues/829
    """

    def __init__(
        self, text: str, **kwargs,
    ):
        """A Warning Alert that renders Markdown

        CSS Styling can be done via the classes 'alert' and 'alert-warning'

        Arguments:
            text {str} -- Some MarkDown text

        """
        if "css_classes" in kwargs:
            if not "alert" in kwargs["css_classes"]:
                kwargs["css_classes"].append("alert")
            if not "alert-warning" in kwargs["css_classes"]:
                kwargs["css_classes"].append("alert-warning")
        else:
            kwargs["css_classes"] = [
                "alert",
                "alert-warning",
            ]
        if "margin" not in kwargs:
            kwargs["margin"] = (0, 0, 25, 0)
        if "sizing_mode" not in kwargs:
            kwargs["sizing_mode"] = "stretch_width"

        super().__init__(
            text, **kwargs,
        )

    raw_css = """
.bk.alert {
    position: relative;
    padding: 0.75rem 1.25rem;
    border: 1px solid transparent;
    border-radius: 0.25rem;
}

.bk.alert-warning {
    color: #856404;
    background-color: #fff3cd;
    border-color: #ffeeba;
}

.bk.alert-warning hr {
    border-top-color: #ffe8a1;
}
"""


class ErrorAlert(pn.pane.Markdown):
    """An Error Alert that renders Markdown

    CSS Styling can be done via the classes 'alert' and 'alert-error'. See the raw_css attribute

    Don't set sizing_mode="stretch_width" as this will result in problems.
    See [Issue 829](https://github.com/holoviz/panel/issues/829)
    """

    def __init__(
        self, text: str, **kwargs,
    ):
        """An Error Alert that renders Markdown

        CSS Styling can be done via the classes 'alert' and 'alert-error'

        Arguments:
            text {str} -- Some MarkDown text

        """
        if "css_classes" in kwargs:
            if not "alert" in kwargs["css_classes"]:
                kwargs["css_classes"].append("alert")
            if not "alert-error" in kwargs["css_classes"]:
                kwargs["css_classes"].append("alert-error")
        else:
            kwargs["css_classes"] = [
                "alert",
                "alert-error",
            ]

        if "margin" not in kwargs:
            kwargs["margin"] = (0, 0, 25, 0)
        if "sizing_mode" not in kwargs:
            kwargs["sizing_mode"] = "stretch_width"

        super().__init__(
            text, **kwargs,
        )

    raw_css = """
.bk.alert {
    position: relative;
    padding: 0.75rem 1.25rem;
    border: 1px solid transparent;
    border-radius: 0.25rem;
}

.bk.alert-error {
    color: #721c24;
    background-color: #f8d7da;
    border-color: #f5c6cb;
}

.bk.alert-error hr {
    border-top-color: #f1b0b7;
}
"""



"""In this module we define all Resource Tags and exposes them via the TAGS list."""
from awesome_panel.shared.models import Tag

# Please keep the Tags sorted alphabetically
ALTERNATIVE = Tag(name="Alternatives")
APP = Tag("App")
APP_IN_GALLERY = Tag(name="App In Gallery")
ARTICLE = Tag(name="Article")
AWESOME_PANEL_ORG = Tag(name="Awesome-panel.org")
AWESOMEPANEL_EXPRESS = Tag(name="awesome_panel.express")
CODE = Tag("Code")
DEPLOYMENT = Tag(name="Deployment")
INSPIRATION = Tag(name="Inspiration")
FINANCE = Tag(name="Finance")
IMAGE_RECOGNITION = Tag(name="Image Recognition")
MACHINE_LEARNING = Tag(name="Machine Learning")
NLP = Tag(name="NLP")
SOCIAL = Tag(name="Social")
PANEL = Tag(name="Panel")
SISTER_SITES = Tag(name="Sister Sites")
TECHNICAL = Tag(name="Technical")
TUTORIAL = Tag(name="Tutorial")
VIDEO = Tag(name="Video")
VOILA = Tag(name="Voila")
VTK = Tag(name="VTK")

# Please keep this list sorted alphabetically
TAGS = [
    ALTERNATIVE,
    APP,
    APP_IN_GALLERY,
    ARTICLE,
    AWESOMEPANEL_EXPRESS,
    AWESOME_PANEL_ORG,
    CODE,
    DEPLOYMENT,
    INSPIRATION,
    FINANCE,
    IMAGE_RECOGNITION,
    MACHINE_LEARNING,
    NLP,
    SOCIAL,
    PANEL,
    TECHNICAL,
    TUTORIAL,
    VIDEO,
    VOILA,
    VTK,
]



"""Setup file. The package can be installed for development by running

pip install -e .

or similar where . is replaced by the path to package root
"""
import pathlib

from setuptools import find_packages, setup

README_FILE_PATH = pathlib.Path(__file__).parent / "README.md"
with open(README_FILE_PATH) as f:
    README = f.read()

s = setup(  # pylint: disable=invalid-name
    name="awesome-panel",
    version="20200511.1",
    license="MIT",
    description="""This package supports the Awesome Panel Project and
    provides highly experimental features!""",
    long_description_content_type="text/markdown",
    long_description=README,
    url="https://github.com/MarcSkovMadsen/awesome-panel",
    author="Marc Skov Madsen",
    author_email="marc.skov.madsen@gmail.com",
    # package_data={"awesome_panel": ["py.typed"]},
    include_package_data=True,
    packages=find_packages(include=["awesome_panel", "awesome_panel.*"]),
    install_requires=["panel"],
    python_requires=">= 3.7",
    zip_safe=False,
)



"""An App Template based on Bootstrap with a header, sidebar and main section"""
import pathlib

import panel as pn

import awesome_panel.express as pnx
from awesome_panel.express.assets import SCROLLBAR_PANEL_EXPRESS_CSS

BOOTSTRAP_DASHBOARD_CSS = pathlib.Path(__file__).parent / "bootstrap_dashboard.css"
BOOTSTRAP_DASHBOARD_TEMPLATE = pathlib.Path(__file__).parent / "bootstrap_dashboard.html"

HEADER_HEIGHT = 58
SIDEBAR_WIDTH = 200

# Hack to make dynamically adding plotly work:
# See https://github.com/holoviz/panel/issues/840
pn.extension("plotly")


class BootstrapDashboardTemplate(pn.Template):
    """A Basic App Template"""

    def __init__(
        self, app_title: str = "App Name", app_url="#",
    ):
        pn.config.raw_css.append(BOOTSTRAP_DASHBOARD_CSS.read_text())
        pn.config.raw_css.append(SCROLLBAR_PANEL_EXPRESS_CSS.read_text())
        pnx.bootstrap.extend()
        pnx.fontawesome.extend()
        template = BOOTSTRAP_DASHBOARD_TEMPLATE.read_text()

        app_title = pn.Row(
            pn.pane.Markdown(f"[{app_title}]({app_url})", css_classes=["app-title"],),
            width=SIDEBAR_WIDTH,
            sizing_mode="stretch_height",
        )
        header = pn.Row(
            app_title, pn.layout.HSpacer(), sizing_mode="stretch_width", height=HEADER_HEIGHT,
        )
        top_spacer = pn.layout.HSpacer(height=15)
        self.header = header
        self.sidebar = pn.Column(top_spacer, height_policy="max", width=SIDEBAR_WIDTH,)
        self.main = pn.Column(sizing_mode="stretch_width", margin=(25, 50, 25, 50,),)

        items = {
            "header": header,
            "sidebar": self.sidebar,
            "main": self.main,
        }
        super().__init__(
            template=template, items=items,
        )



"""In this module we define all Resources (except apps in the gallery) and exposes
them via the RESOURCES list.
"""
from awesome_panel.database import authors, tags
from awesome_panel.database.apps_in_gallery import APPS_IN_GALLERY
from awesome_panel.database.settings import THUMBNAILS_ROOT

# pylint: disable=line-too-long
from awesome_panel.shared.models import Resource

# panel FILE ROOTS

RESOURCES = [
    Resource(
        name="Color Dropper App",
        url="http://colordropper.herokuapp.com/colordropper",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.APP],
        author=authors.ANDREW_HUANG,
    ),
    Resource(
        name="A tour (of a small part) of the Python visualization landscape",
        url="https://indico.cern.ch/event/833895/contributions/3577846/attachments/1928191/3205023/PyHEP2019_slides.pdf",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.ARTICLE],
        author=authors.PHILIPP_RUDIGER,
    ),
    Resource(
        name="Building Dashboards. Introduction to Data Analysis in Biological Sciences.",
        url="https://xavartley.github.io/#panel/vtk_examples/Gallery_VTK.html",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.APP, tags.INSPIRATION],
        author=authors.JUSTIN_BOIS,
    ),
    Resource(
        name="VTK Examples by xavArtley",
        url="http://bebi103.caltech.edu.s3-website-us-east-1.amazonaws.com/2019a/content/recitations/recitation_05/dashboards.html",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.TUTORIAL, tags.INSPIRATION, tags.VTK],
        author=authors.XAVARTLEY,
    ),
    Resource(
        name="XrViz",
        url="https://github.com/intake/xrviz",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.APP, tags.CODE, tags.INSPIRATION,],
        author=authors.INTAKE,
    ),
    Resource(
        name="Information is Beautiful",
        url=(
            "https://towardsdatascience.com/how-to-build-a-time-series-dashboard-in-python-with-"
            "panel-altair-and-a-jupyter-notebook-c0ed40f02289"
        ),
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.TUTORIAL, tags.ARTICLE,],
        author=authors.BENJAMIN_COOLEY,
    ),
    Resource(
        name="Information is Beautiful",
        url="https://informationisbeautiful.net/",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.INSPIRATION],
        author=authors.INFORMATIONISBEUTIFULL,
    ),
    Resource(
        name="Open Source Directions ep. 29: Panel",
        url="https://www.youtube.com/watch?v=hZOsxmM_wyg",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.VIDEO],
        author=authors.QUANSIGHT,
    ),
    Resource(
        name="Our World in Data",
        url="https://ourworldindata.org/",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.INSPIRATION],
        author=authors.OURWORLDINDATA,
    ),
    Resource(
        name="Panel",
        url="https://panel.pyviz.org/",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.PANEL],
        author=authors.PANEL,
    ),
    Resource(
        name="Discourse",
        url="https://discourse.holoviz.org/c/panel",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.PANEL],
        author=authors.PANEL,
    ),
    Resource(
        name="Getting Started",
        url="https://panel.pyviz.org/getting_started/index.html",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.PANEL],
        author=authors.PANEL,
    ),
    Resource(
        name="User Guide",
        url="https://panel.pyviz.org/user_guide/index.html",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.PANEL],
        author=authors.PANEL,
    ),
    Resource(
        name="Gallery",
        url="https://panel.pyviz.org/gallery/index.html",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.PANEL],
        author=authors.PANEL,
    ),
    Resource(
        name="Reference Gallery",
        url="https://panel.pyviz.org/reference/index.html",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.PANEL],
        author=authors.PANEL,
    ),
    Resource(
        name="GitHub",
        url="https://github.com/holoviz/panel",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.PANEL],
        author=authors.PANEL,
    ),
    Resource(
        name="Announcing Article",
        url="https://medium.com/@philipp.jfr/panel-announcement-2107c2b15f52",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.PANEL],
        author=authors.PHILIPP_RUDIGER,
    ),
    Resource(
        name="Awesome-panel.org",
        url="https://awesome-panel.org",
        thumbnail_path=THUMBNAILS_ROOT + "awesome-panel-org.png?raw=true",
        is_awesome=True,
        tags=[tags.AWESOME_PANEL_ORG],
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Github",
        url="https://github.com/marcskovmadsen/awesome-panel",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.AWESOME_PANEL_ORG],
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Docs",
        url="https://awesome-panel.readthedocs.io/en/latest/",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.AWESOME_PANEL_ORG],
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Docker",
        url="https://hub.docker.com/r/marcskovmadsen/awesome-panel",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.AWESOME_PANEL_ORG],
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="PyPi",
        url="https://pypi.org/project/awesome-panel/",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.AWESOME_PANEL_ORG],
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Dashboards with PyViz Panel for interactive web apps",
        url="https://dmnfarrell.github.io/bioinformatics/pyviz-panel",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.ARTICLE],
        author=authors.DAMIAN_FARRELL,
    ),
    Resource(
        name="Turn any Notebook into a Deployable Dashboard | SciPy 2019 | James Bednar",
        url="https://www.youtube.com/watch?v=L91rd1D6XTA&t=274s",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.VIDEO, tags.TUTORIAL,],
        author=authors.JAMES_BEDNAR,
    ),
    Resource(
        name="Turn any notebook into a deployable dashboard|PyData Berlin 2019",
        url="https://www.youtube.com/watch?v=Ohr29FJjBi0&list=PLGVZCDnMOq0pNHTYo3i56zYU-Tdw5Uguw",
        thumbnail_path=THUMBNAILS_ROOT + "pydataberlin2019.png?raw=true",
        is_awesome=True,
        tags=[tags.VIDEO, tags.TUTORIAL,],
        author=authors.PHILIPP_RUDIGER,
    ),
    Resource(
        name="Visualize any Data Easily, from Notebooks to Dashboards",
        url="https://www.youtube.com/watch?v=7deGS4IPAQ0&t=1326s",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.VIDEO, tags.TUTORIAL,],
        author=authors.JAMES_BEDNAR,
    ),
    Resource(
        name="HoloViz.org - Awesome Resources and Tutorials",
        url="http://holoviz.org/index.html",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.TUTORIAL],
        author=authors.HOLOVIZ,
    ),
    Resource(
        name="awesome-streamlit.org",
        url="https://awesome-streamlit.org",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.SISTER_SITES],
        author=authors.MARC_SKOV_MADSEN,
    ),
    Resource(
        name="Bokeh",
        url="https://bokeh.pydata.org/en/latest/index.html",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.ALTERNATIVE],
        author=authors.BOKEH,
    ),
    Resource(
        name="Jupyter Voila",
        url="https://blog.jupyter.org/and-voil%C3%A0-f6a2c08a4a93",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.ALTERNATIVE],
        author=authors.VOILA,
    ),
    Resource(
        name="Plotly Dash",
        url="https://plot.ly/dash/",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.ALTERNATIVE],
        author=authors.PLOTLY,
    ),
    Resource(
        name="Streamlit",
        url="https://streamlit.io",
        thumbnail_path=THUMBNAILS_ROOT + "",
        is_awesome=True,
        tags=[tags.ALTERNATIVE],
        author=authors.STREAMLIT,
    ),
] + APPS_IN_GALLERY

TAGS = []
for resource in RESOURCES:
    for tag in resource.tags:
        TAGS.append(tag)
TAGS = sorted(list(set(TAGS)))



"""In this module we define all Authors of resources or apps in the gallery and exposes
them via the AUTHORS list"""
from awesome_panel.shared.models import Author

# Authors
ANDREW_HUANG = Author(
    name="Andrew Huang",
    url="https://twitter.com/iateandrew1",
    github_url="https://github.com/ahuang11",
    github_avatar_url=(
        "https://avatars3.githubusercontent.com/u/"
        "15331990?s=400&u=6f855e2f82d576fd22110f2ac9b1b7ec4825d916&v=4"
    ),
)
AWESOME_PANEL_ORG = Author(
    name="Awesome-panel.org",
    url="https://awesome-panel.org",
    github_url="https://github.com/marcskovmadsen",
    github_avatar_url="https://avatars0.githubusercontent.com/u/24277277?s=400&v=4",
)
BENJAMIN_COOLEY = Author(
    name="Bokeh",
    url="https://bendoesdata.github.io/",
    github_url="https://github.com/bendoesdata",
    github_avatar_url="https://avatars3.githubusercontent.com/u/8440965",
)
BOKEH = Author(
    name="Bokeh",
    url="https://docs.bokeh.org/en/latest/ ",
    github_url="https://github.com/bokeh",
    github_avatar_url="https://avatars3.githubusercontent.com/u/8440965",
)
DASH = Author(
    name="Dash",
    url="https://plot.ly/dash/",
    github_url="https://github.com/plotly/dash",
    github_avatar_url="https://avatars2.githubusercontent.com/u/5997976",
)
HOLOVIZ = Author(
    name="HoloViz",
    url="http://holoviz.org/",
    github_url="https://github.com/holoviz",
    github_avatar_url="https://avatars2.githubusercontent.com/u/51678735",
)
INFORMATIONISBEUTIFULL = Author(
    name="Information is beautiful",
    url="https://informationisbeautiful.net/",
    github_url="",
    github_avatar_url="",
)
INTAKE = Author(
    name="Intake",
    url="https://github.com/intake",
    github_url="https://github.com/intake",
    github_avatar_url="https://avatars1.githubusercontent.com/u/1469464",
)
JAMES_BEDNAR = Author(
    name="James A. Bednar",
    url="https://github.com/jbednar",
    github_url="https://github.com/jbednar",
    github_avatar_url="https://avatars0.githubusercontent.com/u/1695496",
)
JUSTIN_BOIS = Author(
    name="Justin Bois",
    url="http://bois.caltech.edu/",
    github_url="https://github.com/JUSTIN_BOIS",
    github_avatar_url="https://avatars3.githubusercontent.com/u/1873517",
)
MARC_SKOV_MADSEN = Author(
    name="Marc Skov Madsen",
    url="https://datamodelsanalytics.com",
    github_url="https://github.com/marcskovmadsen",
    github_avatar_url="https://avatars0.githubusercontent.com/u/42288570",
)
OURWORLDINDATA = Author(
    name="Our World in Data",
    url="https://ourworldindata.org/",
    github_url="https://github.com/owid",
    github_avatar_url="https://avatars3.githubusercontent.com/u/14187135",
)
PANEL = Author(
    name="panel",
    url="https://panel.pyviz.org/",
    github_url="https://github.com/holoviz/",
    github_avatar_url="https://avatars2.githubusercontent.com/u/51678735",
)
PHILIPP_RUDIGER = Author(
    name="Philipp Rudiger",
    url="http://philippjfr.com/",
    github_url="https://github.com/philippjfr",
    github_avatar_url="https://avatars0.githubusercontent.com/u/1550771",
)
PLOTLY = Author(
    name="Plotly",
    url="https://plot.ly/",
    github_url="https://github.com/plotly",
    github_avatar_url="https://avatars2.githubusercontent.com/u/5997976",
)
QUANSIGHT = Author(
    name="Quansight",
    url="https://www.quansight.com/",
    github_url="https://github.com/Quansight",
    github_avatar_url="https://avatars1.githubusercontent.com/u/34879953",
)
STREAMLIT = Author(
    name="Streamlit",
    url="https://streamlit.io/",
    github_url="https://github.com/streamlit",
    github_avatar_url="https://avatars1.githubusercontent.com/u/45109972",
)
VOILA = Author(
    name="Voila",
    url="https://github.com/voila-dashboards",
    github_url="https://github.com/voila-dashboards",
    github_avatar_url="https://avatars1.githubusercontent.com/u/55792893?s=200&v=4",
)
DAMIAN_FARRELL = Author(
    name="Damien Farrell",
    url="https://dmnfarrell.github.io/",
    github_url="https://github.com/dmnfarrell",
    github_avatar_url="https://avatars0.githubusercontent.com/u/7859189",
)
XAVARTLEY = Author(
    name="xavArtely",
    url="https://github.com/xavArtley",
    github_url="https://github.com/xavArtley",
    github_avatar_url="https://avatars1.githubusercontent.com/u/18531147",
)

AUTHORS = [
    ANDREW_HUANG,
    AWESOME_PANEL_ORG,
    BOKEH,
    DASH,
    DAMIAN_FARRELL,
    HOLOVIZ,
    INTAKE,
    JAMES_BEDNAR,
    MARC_SKOV_MADSEN,
    OURWORLDINDATA,
    PANEL,
    PHILIPP_RUDIGER,
    QUANSIGHT,
    PLOTLY,
    STREAMLIT,
    VOILA,
    XAVARTLEY,
]
